package com.jiuqi.util.zip;

import java.util.zip.ZipException;
import java.util.Vector;
import java.util.Hashtable;

public class ExtraFieldUtils
{
    private static Hashtable implementations;
    
    static {
        ExtraFieldUtils.implementations = new Hashtable();
        final AsiExtraField field = new AsiExtraField();
        register(field.getClass());
    }
    
    public static void register(final Class c) {
        try {
            final ZipExtraField ze = c.newInstance();
            ExtraFieldUtils.implementations.put(ze.getHeaderId(), c);
        }
        catch (final ClassCastException ex) {
            throw new ZipRuntimeException(c + " doesn't implement ZipExtraField");
        }
        catch (final InstantiationException ex2) {
            throw new ZipRuntimeException(c + " is not a concrete class");
        }
        catch (final IllegalAccessException ex3) {
            throw new ZipRuntimeException(c + "'s no-arg constructor is not public");
        }
    }
    
    public static ZipExtraField createExtraField(final ZipShort headerId) throws InstantiationException, IllegalAccessException {
        final Class c = ExtraFieldUtils.implementations.get(headerId);
        if (c != null) {
            return c.newInstance();
        }
        final UnrecognizedExtraField u = new UnrecognizedExtraField();
        u.setHeaderId(headerId);
        return u;
    }
    
    public static ZipExtraField[] parse(final byte[] data) throws ZipException {
        final Vector v = new Vector();
        int start;
        int length;
        for (start = 0; start <= data.length - 4; start += length + 4) {
            final ZipShort headerId = new ZipShort(data, start);
            length = new ZipShort(data, start + 2).getValue();
            if (start + 4 + length > data.length) {
                throw new ZipException("data starting at " + start + " is in unknown format");
            }
            try {
                final ZipExtraField ze = createExtraField(headerId);
                ze.parseFromLocalFileData(data, start + 4, length);
                v.addElement(ze);
            }
            catch (final InstantiationException ie) {
                throw new ZipException(ie.getMessage());
            }
            catch (final IllegalAccessException iae) {
                throw new ZipException(iae.getMessage());
            }
        }
        if (start != data.length) {
            throw new ZipException("data starting at " + start + " is in unknown format");
        }
        final ZipExtraField[] result = new ZipExtraField[v.size()];
        v.copyInto(result);
        return result;
    }
    
    public static byte[] mergeLocalFileDataData(final ZipExtraField[] data) {
        int sum = 4 * data.length;
        for (int i = 0; i < data.length; ++i) {
            sum += data[i].getLocalFileDataLength().getValue();
        }
        final byte[] result = new byte[sum];
        int start = 0;
        for (int j = 0; j < data.length; ++j) {
            System.arraycopy(data[j].getHeaderId().getBytes(), 0, result, start, 2);
            System.arraycopy(data[j].getLocalFileDataLength().getBytes(), 0, result, start + 2, 2);
            final byte[] local = data[j].getLocalFileDataData();
            System.arraycopy(local, 0, result, start + 4, local.length);
            start += local.length + 4;
        }
        return result;
    }
    
    public static byte[] mergeCentralDirectoryData(final ZipExtraField[] data) {
        int sum = 4 * data.length;
        for (int i = 0; i < data.length; ++i) {
            sum += data[i].getCentralDirectoryLength().getValue();
        }
        final byte[] result = new byte[sum];
        int start = 0;
        for (int j = 0; j < data.length; ++j) {
            System.arraycopy(data[j].getHeaderId().getBytes(), 0, result, start, 2);
            System.arraycopy(data[j].getCentralDirectoryLength().getBytes(), 0, result, start + 2, 2);
            final byte[] local = data[j].getCentralDirectoryData();
            System.arraycopy(local, 0, result, start + 4, local.length);
            start += local.length + 4;
        }
        return result;
    }
}
